@@ -1,20 +1,17 @@
-use std::{convert::TryInto, iter};
+use std::convert::TryInto;
 
-use either::Either;
-use hir::{AsAssocItem, InFile, ModuleDef, Semantics};
+use crate::{
+    display::TryToNav, doc_links::token_as_doc_comment, FilePosition, NavigationTarget, RangeInfo,
+};
+use hir::{AsAssocItem, ModuleDef, Semantics};
 use ide_db::{
     base_db::{AnchoredPath, FileId, FileLoader},
-    defs::{Definition, NameClass, NameRefClass},
+    defs::Definition,
     helpers::pick_best_token,
     RootDatabase,
 };
-use syntax::{ast, match_ast, AstNode, AstToken, SyntaxKind::*, SyntaxToken, TextRange, T};
-
-use crate::{
-    display::{ToNav, TryToNav},
-    doc_links::{doc_attributes, extract_definitions_from_markdown, resolve_doc_path_for_def},
-    FilePosition, NavigationTarget, RangeInfo,
-};
+use itertools::Itertools;
+use syntax::{ast, AstNode, AstToken, SyntaxKind::*, SyntaxToken, TextRange, T};
 
 // Feature: Go to Definition
 //
@@ -31,7 +28,7 @@ pub(crate) fn goto_definition(
     db: &RootDatabase,
     position: FilePosition,
 ) -> Option<RangeInfo<Vec<NavigationTarget>>> {
-    let sema = Semantics::new(db);
+    let sema = &Semantics::new(db);
     let file = sema.parse(position.file_id).syntax().clone();
     let original_token =
         pick_best_token(file.token_at_offset(position.offset), |kind| match kind {
@@ -39,67 +36,56 @@ pub(crate) fn goto_definition(
             kind if kind.is_trivia() => 0,
             _ => 1,
         })?;
-    let token = sema.descend_into_macros(original_token.clone());
-    let parent = token.parent()?;
-    if let Some(_) = ast::Comment::cast(token.clone()) {
-        let (attributes, def) = doc_attributes(&sema, &parent)?;
-
-        let (docs, doc_mapping) = attributes.docs_with_rangemap(db)?;
-        let (_, link, ns) =
-            extract_definitions_from_markdown(docs.as_str()).into_iter().find(|(range, ..)| {
-                doc_mapping.map(*range).map_or(false, |InFile { file_id, value: range }| {
-                    file_id == position.file_id.into() && range.contains(position.offset)
-                })
-            })?;
-        let nav = resolve_doc_path_for_def(db, def, &link, ns)?.try_to_nav(db)?;
-        return Some(RangeInfo::new(original_token.text_range(), vec![nav]));
+    if let Some(doc_comment) = token_as_doc_comment(&original_token) {
+        return doc_comment.get_definition_with_descend_at(sema, position.offset, |def, _, _| {
+            let nav = def.try_to_nav(db)?;
+            Some(RangeInfo::new(original_token.text_range(), vec![nav]))
+        });
     }
-
-    let navs = match_ast! {
-        match parent {
-            ast::NameRef(name_ref) => {
-                reference_definition(&sema, Either::Right(&name_ref))
-            },
-            ast::Name(name) => {
-                match NameClass::classify(&sema, &name)? {
-                    NameClass::Definition(def) | NameClass::ConstReference(def) => {
-                        try_find_trait_item_definition(sema.db, &def).unwrap_or_else(|| def_to_nav(sema.db, def))
-                    }
-                    NameClass::PatFieldShorthand { local_def, field_ref } => {
-                        local_and_field_to_nav(sema.db, local_def, field_ref)
-                    },
+    let navs = sema
+        .descend_into_macros(original_token.clone())
+        .into_iter()
+        .filter_map(|token| {
+            let parent = token.parent()?;
+            if let Some(tt) = ast::TokenTree::cast(parent) {
+                if let x @ Some(_) =
+                    try_lookup_include_path(sema, tt, token.clone(), position.file_id)
+                {
+                    return x;
                 }
-            },
-            ast::Lifetime(lt) => if let Some(name_class) = NameClass::classify_lifetime(&sema, &lt) {
-                match name_class {
-                    NameClass::Definition(def) => def_to_nav(sema.db, def),
-                    _ => return None,
-                }
-            } else {
-                reference_definition(&sema, Either::Left(&lt))
-            },
-            ast::TokenTree(tt) => try_lookup_include_path(sema.db, tt, token, position.file_id)?,
-            _ => return None,
-        }
-    };
+            }
+            Some(
+                Definition::from_token(sema, &token)
+                    .into_iter()
+                    .flat_map(|def| {
+                        try_find_trait_item_definition(sema.db, &def)
+                            .unwrap_or_else(|| def_to_nav(sema.db, def))
+                    })
+                    .collect::<Vec<_>>(),
+            )
+        })
+        .flatten()
+        .unique()
+        .collect::<Vec<NavigationTarget>>();
 
     Some(RangeInfo::new(original_token.text_range(), navs))
 }
 
 fn try_lookup_include_path(
-    db: &RootDatabase,
+    sema: &Semantics<RootDatabase>,
     tt: ast::TokenTree,
     token: SyntaxToken,
     file_id: FileId,
 ) -> Option<Vec<NavigationTarget>> {
-    let path = ast::String::cast(token)?.value()?.into_owned();
+    let token = ast::String::cast(token)?;
+    let path = token.value()?.into_owned();
     let macro_call = tt.syntax().parent().and_then(ast::MacroCall::cast)?;
     let name = macro_call.path()?.segment()?.name_ref()?;
     if !matches!(&*name.text(), "include" | "include_str" | "include_bytes") {
         return None;
     }
-    let file_id = db.resolve_path(AnchoredPath { anchor: file_id, path: &path })?;
-    let size = db.file_text(file_id).len().try_into().ok()?;
+    let file_id = sema.db.resolve_path(AnchoredPath { anchor: file_id, path: &path })?;
+    let size = sema.db.file_text(file_id).len().try_into().ok()?;
     Some(vec![NavigationTarget {
         file_id,
         full_range: TextRange::new(0.into(), size),
@@ -144,37 +130,10 @@ fn try_find_trait_item_definition(
         .map(|it| vec![it])
 }
 
-pub(crate) fn reference_definition(
-    sema: &Semantics<RootDatabase>,
-    name_ref: Either<&ast::Lifetime, &ast::NameRef>,
-) -> Vec<NavigationTarget> {
-    let name_kind = match name_ref.either(
-        |lifetime| NameRefClass::classify_lifetime(sema, lifetime),
-        |name_ref| NameRefClass::classify(sema, name_ref),
-    ) {
-        Some(class) => class,
-        None => return Vec::new(),
-    };
-    match name_kind {
-        NameRefClass::Definition(def) => def_to_nav(sema.db, def),
-        NameRefClass::FieldShorthand { local_ref, field_ref } => {
-            local_and_field_to_nav(sema.db, local_ref, field_ref)
-        }
-    }
-}
-
 fn def_to_nav(db: &RootDatabase, def: Definition) -> Vec<NavigationTarget> {
     def.try_to_nav(db).map(|it| vec![it]).unwrap_or_default()
 }
 
-fn local_and_field_to_nav(
-    db: &RootDatabase,
-    local: hir::Local,
-    field: hir::Field,
-) -> Vec<NavigationTarget> {
-    iter::once(local.to_nav(db)).chain(field.try_to_nav(db)).collect()
-}
-
 #[cfg(test)]
 mod tests {
     use ide_db::base_db::FileRange;
@@ -182,10 +141,11 @@ mod tests {
 
     use crate::fixture;
 
+    #[track_caller]
     fn check(ra_fixture: &str) {
         let (analysis, position, expected) = fixture::annotations(ra_fixture);
         let navs = analysis.goto_definition(position).unwrap().expect("no definition found").info;
-        if navs.len() == 0 {
+        if navs.is_empty() {
             panic!("unresolved reference")
         }
 
@@ -210,6 +170,29 @@ mod tests {
         assert!(navs.is_empty(), "didn't expect this to resolve anywhere: {:?}", navs)
     }
 
+    #[test]
+    fn goto_def_in_mac_call_in_attr_invoc() {
+        check(
+            r#"
+//- proc_macros: identity
+pub struct Struct {
+        // ^^^^^^
+    field: i32,
+}
+
+macro_rules! identity {
+    ($($tt:tt)*) => {$($tt)*};
+}
+
+#[proc_macros::identity]
+fn function() {
+    identity!(Struct$0 { field: 0 });
+}
+
+"#,
+        )
+    }
+
     #[test]
     fn goto_def_for_extern_crate() {
         check(
@@ -1380,6 +1363,69 @@ struct Stwuct;
 impl Twait for Stwuct {
     type IsBad$0 = !;
 }
+"#,
+        );
+    }
+
+    #[test]
+    fn goto_def_derive_input() {
+        check(
+            r#"
+//- minicore:derive
+#[rustc_builtin_macro]
+pub macro Copy {}
+       // ^^^^
+#[derive(Copy$0)]
+struct Foo;
+            "#,
+        );
+        check(
+            r#"
+//- minicore:derive
+mod foo {
+    #[rustc_builtin_macro]
+    pub macro Copy {}
+           // ^^^^
+}
+#[derive(foo::Copy$0)]
+struct Foo;
+            "#,
+        );
+    }
+
+    #[test]
+    fn goto_def_in_macro_multi() {
+        check(
+            r#"
+struct Foo {
+    foo: ()
+  //^^^
+}
+macro_rules! foo {
+    ($ident:ident) => {
+        fn $ident(Foo { $ident }: Foo) {}
+    }
+}
+foo!(foo$0);
+   //^^^
+   //^^^
+"#,
+        );
+        check(
+            r#"
+fn bar() {}
+ //^^^
+struct bar;
+     //^^^
+macro_rules! foo {
+    ($ident:ident) => {
+        fn foo() {
+            let _: $ident = $ident;
+        }
+    }
+}
+
+foo!(bar$0);
 "#,
         );
     }
